/**
 * Tocea Softwares.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 23 mai 2012
 * @copyright Copyright (C) 2010-2012 Tocea
 */

package com.tocea.easycoverage.framework;



import java.util.List;

import junit.framework.TestCase;
import junit.framework.TestResult;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.annotations.SkipAutomaticTests;
import com.tocea.easycoverage.framework.behaviour.api.IClassTest;
import com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes the test instance.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 23 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class SuiteClassTest extends TestCase
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(SuiteClassTest.class);
    private final List<ITestBehaviour>    testBehaviours;
    private final Class<?>                clazz;
    
    
    
    /**
     * Builds a suite class test.
     * 
     * @param _clazz
     * @param _testBehaviours
     */
    @SkipAutomaticTests
    public SuiteClassTest(final Class<?> _clazz, final List<ITestBehaviour> _testBehaviours) {
    
    
        super(_clazz.getCanonicalName());
        this.clazz = _clazz;
        this.testBehaviours = _testBehaviours;
        
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see junit.framework.Test#countTestCases()
     */
    @Override
    public int countTestCases() {
    
    
        return 1;
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see junit.framework.Test#run(junit.framework.TestResult)
     */
    @Override
    public void run(final TestResult _result) {
    
    
        _result.startTest(this);
        
        try {
            if (!this.testingClass()) {
                _result.addError(this, new RuntimeException());
            }
            
            
        } catch (final WeakImplementationException e) {
            _result.addError(this, e);
        } finally {
            _result.endTest(this);
        }
        // _result.stop();
        
        
    }
    
    
    /**
     * This method performs the test of a class
     * 
     * @param clazz
     * @return the test of a class
     */
    public boolean testingClass() throws WeakImplementationException {
    
    
        boolean res = false;
        final SkipAutomaticTests annotation = this.clazz.getAnnotation(SkipAutomaticTests.class);
        if (annotation != null) {
            LOGGER.info("\t[SKIP] {}", this.clazz);
            
            return true;
        }
        LOGGER.info("\t[TEST] {}", this.clazz);
        for (final ITestBehaviour testBehaviour : this.testBehaviours) {
            final IClassTest newInstanceTest = testBehaviour.newTest(this.clazz);
            if (newInstanceTest == null) {
                continue;
            }
            newInstanceTest.performTest();
            res = true;
        }
        return res;
    }
    
    
    @Override
    public String toString() {
    
    
        return "Test of " + this.clazz.getCanonicalName();
    }
}
