/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.behaviour;



import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Modifier;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.IClassTest;
import com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour;
import com.tocea.easycoverage.framework.unit.VariableArgumentInvocationTest;
import com.tocea.easycoverage.framework.utils.Utils;



/**
 * This type describes the instantiation behaviour.
 * Default constructors should never failed, both protected and public constructors.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 12 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class VariableArgumentInvocationBehaviour implements ITestBehaviour
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER                  = LoggerFactory.getLogger(VariableArgumentInvocationBehaviour.class);
    
    /**
     * This field defines the modifiers to exclude methods in the analysis.
     */
    public static int                     excludeMethodsModifiers = Modifier.PRIVATE | Modifier.PROTECTED;
    
    
    
    /**
     * Builds a instantiation behaviour.
     */
    public VariableArgumentInvocationBehaviour() {
    
    
        super();
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour#newInstanceTest(java.lang.Object)
     */
    public IClassTest newInstanceTest(final Object _testedClass) {
    
    
        return new VariableArgumentInvocationTest(_testedClass, excludeMethodsModifiers);
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.ITestBehaviour#newTest(java.lang.Class)
     */
    public IClassTest newTest(final Class<?> _testedClass) {
    
    
        for (final Constructor<?> constructor : Utils.getAccessibleConstructors(_testedClass, excludeMethodsModifiers)) {
            try {
                constructor.setAccessible(true);
                final VariableArgumentInvocationTest simpleInvokationTest =
                        new VariableArgumentInvocationTest(constructor.newInstance(), excludeMethodsModifiers);
                constructor.setAccessible(false);
                
                return simpleInvokationTest;
            } catch (final IllegalArgumentException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final InstantiationException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final IllegalAccessException e) {
                LOGGER.error(e.getMessage(), e);
                
            } catch (final InvocationTargetException e) {
                LOGGER.error(e.getMessage(), e);
                
            }
        }
        return null;
    }
    
    
    @Override
    public String toString() {
    
    
        return "Variable argument invocation behaviour";
    }
}
