/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.Array;
import java.lang.reflect.Method;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.IClassTest;
import com.tocea.easycoverage.framework.utils.MethodImplementationException;
import com.tocea.easycoverage.framework.utils.Utils;
import com.tocea.easycoverage.framework.utils.WeakImplementationException;



/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class VariableArgumentInvocationTest implements IClassTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(VariableArgumentInvocationTest.class);
    private final Object                  testedObject;
    private final int                     excludeMethodsModifiers;
    
    
    
    /**
     * Builds a simple invocation test .
     * 
     * @param _testedObject
     *            the tested class.
     * @param _excludeMethodsModifiers
     */
    public VariableArgumentInvocationTest(final Object _testedObject, final int _excludeMethodsModifiers) {
    
    
        super();
        this.testedObject = _testedObject;
        this.excludeMethodsModifiers = _excludeMethodsModifiers;
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.IClassTest#performTest()
     */
    public void performTest() throws WeakImplementationException {
    
    
        if (Utils.shouldSkip(this.testedObject.getClass())) {
            return;
        }
        LOGGER.debug("Processing instantiation of the class {} if any constructors available");
        for (final Method method : Utils.getMethods(this.testedObject, this.excludeMethodsModifiers)) {
            if (Utils.shouldSkip(method)) {
                continue;
            }
            
            LOGGER.debug("Invokating the method {}", method);
            method.setAccessible(true);
            
            
            if (!method.isVarArgs()) {
                return;
            }
            try {
                final Class[] parameterTypes = method.getParameterTypes();
                final Class class1 = parameterTypes[parameterTypes.length - 1].getComponentType();
                
                
                method.invoke(this.testedObject, new Object[] {
                    Array.newInstance(class1, 0) });
                
                final Object array1 = Array.newInstance(class1, 1);
                final Object array2 = Array.newInstance(class1, 2);
                Array.set(array1, 0, class1.newInstance());
                Array.set(array2, 0, class1.newInstance());
                Array.set(array2, 1, class1.newInstance());
                final Object[] args = new Object[] {
                    
                    array1 };
                method.invoke(this.testedObject, args);
                method.invoke(this.testedObject, new Object[] {
                    
                    array2 });
            } catch (final Throwable e) {
                throw new MethodImplementationException(e, method, VariableArgumentInvocationTest.class.getCanonicalName());
            }
            method.setAccessible(false);
        }
        
        
    }
    
}
